@@ -28,6 +28,19 @@ mapping-orange:
   b: 0
 """
 
+bomb_yaml = """
+lol1: &lol1 "lol"
+lol2: &lol2 [*lol1,*lol1,*lol1,*lol1,*lol1,*lol1,*lol1,*lol1,*lol1]
+lol3: &lol3 [*lol2,*lol2,*lol2,*lol2,*lol2,*lol2,*lol2,*lol2,*lol2]
+lol4: &lol4 [*lol3,*lol3,*lol3,*lol3,*lol3,*lol3,*lol3,*lol3,*lol3]
+lol5: &lol5 [*lol4,*lol4,*lol4,*lol4,*lol4,*lol4,*lol4,*lol4,*lol4]
+lol6: &lol6 [*lol5,*lol5,*lol5,*lol5,*lol5,*lol5,*lol5,*lol5,*lol5]
+lol7: &lol7 [*lol6,*lol6,*lol6,*lol6,*lol6,*lol6,*lol6,*lol6,*lol6]
+lol8: &lol8 [*lol7,*lol7,*lol7,*lol7,*lol7,*lol7,*lol7,*lol7,*lol7]
+lol9: &lol9 [*lol8,*lol8,*lol8,*lol8,*lol8,*lol8,*lol8,*lol8,*lol8]
+lol10: &lol10 [*lol9,*lol9,*lol9,*lol9,*lol9,*lol9,*lol9,*lol9,*lol9]
+"""
+
 class TestYq(unittest.TestCase):
     def run_yq(self, input_data, args, expect_exit_codes={os.EX_OK}, input_format="yaml"):
         stdin, stdout = sys.stdin, sys.stdout
@@ -64,9 +77,10 @@ class TestYq(unittest.TestCase):
     def test_yq_err(self):
         err = ('yq: Error running jq: ScannerError: while scanning for the next token\nfound character \'%\' that '
                'cannot start any token\n  in "<file>", line 1, column 3.')
-        self.run_yq("- %", ["."], expect_exit_codes={err, 2})
+        err2 = ('yq: Error running jq: ScannerError: while scanning for the next token\nfound character that '
+                'cannot start any token\n  in "<file>", line 1, column 3.')
+        self.run_yq("- %", ["."], expect_exit_codes={err, err2, 2})
 
-    @unittest.skipIf(sys.version_info < (3, 5), "Skipping test incompatible with Python 2")
     def test_yq_arg_handling(self):
         from unittest import mock
 
@@ -77,7 +91,6 @@ class TestYq(unittest.TestCase):
         unusable_tty_input = mock.Mock()
         unusable_tty_input.isatty = mock.Mock(return_value=True)
 
-        self.run_yq("{}", [], expect_exit_codes={0} if sys.stdin.isatty() else {2})
         self.run_yq("{}", ["."])
         self.run_yq(unusable_non_tty_input, [".", test_doc])
         self.run_yq(unusable_non_tty_input, [".", test_doc, test_doc])
@@ -136,13 +149,12 @@ class TestYq(unittest.TestCase):
 
     def test_datetimes(self):
         self.assertEqual(self.run_yq("- 2016-12-20T22:07:36Z\n", ["."]), "")
-        if yaml.__version__ < '5.3':
-            self.assertEqual(self.run_yq("- 2016-12-20T22:07:36Z\n", ["-y", "."]), "- '2016-12-20T22:07:36'\n")
-        else:
-            self.assertEqual(self.run_yq("- 2016-12-20T22:07:36Z\n", ["-y", "."]), "- '2016-12-20T22:07:36+00:00'\n")
-
+        self.assertEqual(self.run_yq("- 2016-12-20T22:07:36Z\n", ["-y", "."]), "- '2016-12-20T22:07:36Z'\n")
+        self.assertEqual(self.run_yq("- 2016-12-20T22:07:36Z\n", ["-y", "--yml-out-ver=1.2", "."]),
+                         "- 2016-12-20T22:07:36Z\n")
         self.assertEqual(self.run_yq("2016-12-20", ["."]), "")
         self.assertEqual(self.run_yq("2016-12-20", ["-y", "."]), "'2016-12-20'\n")
+        self.assertEqual(self.run_yq("2016-12-20", ["-y", "--yml-out-ver=1.2", "."]), "2016-12-20\n...\n")
 
     def test_unrecognized_tags(self):
         self.assertEqual(self.run_yq("!!foo bar\n", ["."]), "")
@@ -159,7 +171,6 @@ class TestYq(unittest.TestCase):
         with io.open(cfn_filename) as fh:
             self.assertEqual(self.run_yq("", ["-Y", ".", cfn_filename]), fh.read())
 
-    @unittest.skipIf(sys.version_info < (3, 5), "Skipping feature incompatible with Python 2")
     def test_in_place(self):
         with tempfile.NamedTemporaryFile() as tf, tempfile.NamedTemporaryFile() as tf2:
             tf.write(b"- foo\n- bar\n")
@@ -183,14 +194,6 @@ class TestYq(unittest.TestCase):
         self.assertTrue(self.run_yq("", ["-y", "--explicit-start", ".", test_doc]).startswith("---"))
         self.assertTrue(self.run_yq("", ["-y", "--explicit-end", ".", test_doc]).endswith("...\n"))
 
-    @unittest.expectedFailure
-    def test_times(self):
-        """
-        Timestamps are parsed as sexagesimals in YAML 1.1 but not 1.2. No PyYAML support for YAML 1.2 yet. See issue 10
-        """
-        self.assertEqual(self.run_yq("11:12:13", ["."]), "")
-        self.assertEqual(self.run_yq("11:12:13", ["-y", "."]), "'11:12:13'\n")
-
     def test_xq(self):
         self.assertEqual(self.run_yq("<foo/>", ["."], input_format="xml"), "")
         self.assertEqual(self.run_yq("<foo/>", ["-x", ".foo.x=1"], input_format="xml"),
@@ -231,8 +234,6 @@ class TestYq(unittest.TestCase):
         self.assertEqual(self.run_yq("[foo]\nbar = 1", ["."], input_format="toml"), "")
         self.assertEqual(self.run_yq("[foo]\nbar = 1", ["-t", ".foo"], input_format="toml"), "bar = 1\n")
 
-    @unittest.skipIf(sys.version_info < (3, 5),
-                     "argparse option abbreviation interferes with opt passthrough, can't be disabled until Python 3.5")
     def test_abbrev_opt_collisions(self):
         with tempfile.TemporaryFile() as tf, tempfile.TemporaryFile() as tf2:
             self.assertEqual(
@@ -240,5 +241,52 @@ class TestYq(unittest.TestCase):
                 "true\n...\n"
             )
 
+    def test_entity_expansion_defense(self):
+        self.run_yq(bomb_yaml, ["."], expect_exit_codes=["yq: Error: detected unsafe YAML entity expansion"])
+
+    def test_yaml_type_tags(self):
+        bin_yaml = "example: !!binary Zm9vYmFyCg=="
+        self.assertEqual(self.run_yq(bin_yaml, ["."]), "")
+        self.assertEqual(self.run_yq(bin_yaml, ["-y", "."]), "example: Zm9vYmFyCg==\n")
+        set_yaml = "example: !!set { Boston Red Sox, Detroit Tigers, New York Yankees }"
+        self.assertEqual(self.run_yq(set_yaml, ["."]), "")
+        self.assertEqual(
+            self.run_yq(set_yaml, ["-y", "."]),
+            "example:\n  Boston Red Sox: null\n  Detroit Tigers: null\n  New York Yankees: null\n"
+        )
+
+    def test_yaml_merge(self):
+        self.assertEqual(self.run_yq("a: &b\n  c: d\ne:\n  <<: *b\n  g: h", ["-y", "."]),
+                         "a:\n  c: d\ne:\n  c: d\n  g: h\n")
+
+    def test_yaml_1_2(self):
+        self.assertEqual(self.run_yq("11:12:13", ["."]), "")
+        self.assertEqual(self.run_yq("11:12:13", ["-y", "."]), "'11:12:13'\n")
+
+        self.assertEqual(self.run_yq("on: 12:34:56", ["-y", "."]), "'on': '12:34:56'\n")
+        self.assertEqual(self.run_yq("on: 12:34:56", ["-y", "--yml-out-ver=1.2", "."]), "on: 12:34:56\n")
+
+        self.assertEqual(self.run_yq("2022-02-22", ["-y", "."]), "'2022-02-22'\n")
+        self.assertEqual(self.run_yq("2022-02-22", ["-y", "--yml-out-ver=1.2", "."]), "2022-02-22\n...\n")
+
+        self.assertEqual(self.run_yq("0b1010_0111", ["-y", "."]), "'0b1010_0111'\n")
+        self.assertEqual(self.run_yq("0b1010_0111", ["-y", "--yml-out-ver=1.2", "."]), "0b1010_0111\n...\n")
+
+        self.assertEqual(self.run_yq("0x_0A_74_AE", ["-y", "."]), "'0x_0A_74_AE'\n")
+        self.assertEqual(self.run_yq("0x_0A_74_AE", ["-y", "--yml-out-ver=1.2", "."]), "0x_0A_74_AE\n...\n")
+
+        self.assertEqual(self.run_yq("+685_230", ["-y", "."]), "'+685_230'\n")
+        self.assertEqual(self.run_yq("+685_230", ["-y", "--yml-out-ver=1.2", "."]), "+685_230\n...\n")
+
+        self.assertEqual(self.run_yq("+12345", ["-y", "."]), "12345\n...\n")
+
+    def test_yaml_1_1_octals(self):
+        self.assertEqual(self.run_yq("on: -012345", ["-y", "."]), "'on': -5349\n")
+
+    @unittest.expectedFailure
+    def test_yaml_1_2_octals(self):
+        """YAML 1.2 octals not yet implemented"""
+        self.assertEqual(self.run_yq("on: -012345", ["-y", "--yml-out-ver=1.2", "."]), "on: -12345\n")
+
 if __name__ == '__main__':
     unittest.main()
